{
Copyright (C) 2000, 2001 Eike Lange

Author: Eike Lange <eike.lange@uni-essen.de>

postgres-Unit

This unit implements a set of Funktions for accessing 
PostgreSql. 

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation, version 2.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.

You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA.
}

unit postgres;

interface

uses gpc;
	
const
	{ ConnStatusType }
	CONNECTION_OK                = 0;
	CONNECTION_BAD               = 1;
	CONNECTION_STARTED           = 2;
	CONNECTION_MADE              = 3;
	CONNECTION_AWAITING_RESPONSE = 4;
	CONNECTION_AUTH_OK           = 5;
	CONNECTION_SETENV            = 6;
	
	{ PostgresPollingStatusType, used for PQConnectPoll, PQResetPoll }
	PGRES_POLLING_FAILED   = 0;
	PGRES_POLLING_READING  = 1;
	PGRES_POLLING_WRITING  = 2;
	PGRES_POLLING_OK       = 3;
	PGRES_POLLING_ACTIVE   = 4;

	{ ExecStatusType }
	PGRES_EMPTY_QUERY      = 0;
	PGRES_COMMAND_OK       = 1;
	PGRES_TUPLES_OK        = 2;
	PGRES_COPY_OUT         = 3;
	PGRES_COPY_IN          = 4;
	PGRES_BAD_RESPONSE     = 5;
	PGRES_NONFATAL_ERROR   = 6;
	PGRES_FATAL_ERROR      = 7;

  { See postgres_ext.h for details }
	NAMEDATALEN=32;

type
	PGConn = Pointer;
	PGResult = Pointer;
	PGString = String(200);
	PGLongString = String(1024);
	Oid = Cardinal;
	
	PPQConnInfoOption = ^PQConnInfoOption;
	PQConnInfoOption = record
		pqkeyword, pqenvvar, pqcompiled, pqval, pqlabel, pqdispchar : CString;
		pqdispsize : Integer;
	end;
	
	{ For PGNotifies }
	PPGNotify = ^PGNotify;
	PGNotify = record
		relname : String(NAMEDATALEN);
		be_pid : Integer;
	end;

	{ For PQPrint }
	PPQPrintOpt = ^PQPrintOpt;
	PQPrintOpt = record
		header, align, standard, html3, expanded, pager : Boolean;
		fieldSep : CString;
		tableOpt, caption : CString;
		fieldName : Pointer;
	end;


{ *** Large Objects Access Routines *** }

function LoClose(conn:PGConn; var f:AnyFile):Integer;

function LoCreat(conn:PGConn; mode:Integer):Oid;
asmname 'lo_creat';

function LoImport(conn:PGConn; filename:CString):Oid;
asmname 'lo_import';

function LoExport(conn:PGConn; lObjId:Oid; filename:CString):Integer;
asmname 'lo_export';

function LoOpen(conn:PGConn; lObjId:Oid; mode:Integer):Integer;
asmname 'lo_open';

function LoRead(conn:PGConn; var f:AnyFile; buf:PChar; len:Integer):Integer;

function LoLseek(conn:PGConn; var f:AnyFile; offset,whence:Integer):Integer;

function LoTell(conn:PGConn; var f:AnyFile):Integer;

function LoUnlink(conn:PGConn; lObjId:Oid):Integer;
asmname 'lo_unlink';

function LoWrite(conn:PGConn; var f:AnyFile; buf:PChar; len:Integer):Integer;

{ *** from libpq-fe.h *** }
	
function PQBackendPID(con : PGConn) : Integer;
asmname '_p_pqbackend_pid';

function PQBinaryTuples(res : PGResult) : Boolean;

procedure PQClear(res : PGResult);
asmname '_p_pqclear';

{ Do not know, what this function does, because there is no
ducumentation in postgres-programmers-manual, always returns -1}
function PQClientEncoding(con : PGConn) : Integer;
asmname '_p_pqclientencoding';

function PQCmdStatus(res : PGResult) : PGString;

function PQCmdTuples(res : PGResult) : PGString;

function PQConnDefaults : PPQConnInfoOption;
asmname '_p_pqconndefaults';

function PQConnectDb(coninfo : CString) : PGConn;
asmname '_p_pqconnect_db';

{ For asyncronous connection. Returns 
PostgresPollingStatusType (constants above) }
function PQConnectPoll(con : PGConn) : Integer;
asmname '_p_pqconnect_poll';

{ Make a non-blocking connection to server
Example: connection := PQConnectStart('user=proger dbname=test');}
function PQConnectStart(coninfo : CString) : PGConn;
asmname '_p_pqconnect_start';

{ Frees the Pointer, created by PQConnDefaults }
procedure PQConnInfoFree( co : PPQConnInfoOption );
asmname '_p_pqconninfo_free';

function PQConsumeInput(con : PGConn) : Boolean;

function PQDb(con : PGConn) : PGString;

procedure PQDisplayTuples(res : PGResult; var aFile : Text; 
													fillAlign : Boolean; fieldSep : CString;
													printHeader, quiet : Boolean);

{ function for "COPY" }
function PQEndCopy(con : PGConn) : Integer;
asmname '_p_pqendcopy';
													
{ Get encoding-Id from PGCLIENTENCODING }
function PQEnv2Encoding : Integer;
asmname '_p_pqenv2encoding';

function PQErrorMessage(con : PGConn) : PGString;

function PQExec(con : PGConn; sql : CString) : PGResult;
asmname '_p_pqexec';
	
procedure PQFinish(con : PGConn);
asmname '_p_pqfinish';

function PQFlush(con : PGConn) : Boolean;

function PQFMod(res : PGResult; fnum : Integer) : Integer;
asmname '_p_pqfmod';

function PQFName(res : PGResult; findex : Integer) : PGString;

function PQFNumber(res : PGResult; field : CString) : Integer;
asmname '_p_pqfnumber';

function PQFSize(res : PGResult; findex : Integer) : Integer;
asmname '_p_pqfsize';

function PQFType(res : PGResult; fnum : Integer) : Oid;
asmname '_p_pqftype';

function PQGetIsNull(res : PGResult; tupNum, fieldNum : Integer) : Boolean;

function PQGetLength(res : PGResult; tupNum, fieldNum : Integer) : Integer;
asmname '_p_pqgetlength';

{ function for "COPY" }
function PQGetLine(con : PGConn; s : CString; len : Integer) : Integer;
asmname '_p_pqgetline';

{ function for "COPY" }
function PQGetLineAsync(con : PGConn; buf : CString; buflen : Integer) : Integer;
asmname '_p_pqgetline_async';

function PQGetResult(con : PGconn) : PGResult;
asmname '_p_pq_get_result';

function PQGetValue(res : PGResult; tupNum, fieldNum : Integer) : PGString;

function PQHost(con : PGConn) : PGString;

function PQIsBusy(con : PGConn) : Boolean;

{ Returns True, if connection is in non-blocking mode. But this
function _always_ returns false}
function PQIsNonblocking(con : PGConn) : Boolean;

{ status is one of ExecStatusType (see CONST above) }
function PQMakeEmptyPGResult(con : PGConn; status : Integer) : PGResult;
asmname '_p_pqmake_empty_pgresult';

{ Length of Multibyte-String }
function PQMbLen(s : CString; encoding : Integer) : Integer;
asmname '_p_pqmblen';

function PQNFields(res : PGResult) : Integer;
asmname '_p_pqnfields';

function PQNotifies(con : PGConn) : PPGNotify;

function PQNTuples(res : PGResult) : Integer;
asmname '_p_pqntuples';

function PQOidStatus(res : PGResult) : PGString;

{ returns id of last touple inserted or 0, if no
INSERT-SQL-command was last}
function PQOidValue(res : PGResult) : Oid;
asmname '_p_pqoidvalue';

function PQOptions(con : PGConn) : PGString;

function PQPass(con : PGConn) : PGString;

function PQPort(con : PGConn) : PGString;

procedure PQPrint(var aFile : Text; res : PGResult; ppo : PPQPrintOpt);

{ Prints to stdout, if fd =1 else to stdout }
procedure PQPrintTuples(res : PGResult; var  aFile : Text; 
													PrintAttrName, TerseOutput : Boolean;
													Width : Integer);

{ function for "COPY" }
function PQPutLine(con : PGConn; s : CString) : Integer;
asmname '_p_pqputline';

{ function for "COPY" }
function PQPutNBytes(con : PGConn; buffer : CString; buflen : Integer) : Integer;
asmname '_p_pqputnbytes';

function PQRequestCancel(con : PGconn) : Boolean;

procedure PQReset(con : PGConn);
asmname '_p_pqreset';

{ Reset communication, restarting, non-blocking }
function PQResetPoll(con : PGConn) : Integer;
asmname '_p_pqreset_poll';

{ Reset communication, restarting, non-blocking,
returns true on success }
function PQResetStart(con : PGConn) : Boolean;

{ Returns a string, describing PQResultStatus }
function PQResStatus(status : Integer) : PGLongString;

function PQResultStatus(res : PGResult) : Integer;
asmname '_p_pqresultstatus';

function PQResultErrorMessage(res : PGResult) : PGString;

function PQSendQuery(con : PGConn; query : CString) : Boolean;

{ Do not know, what this function does, because there is no
ducumentation in postgres-programmers-manual}
function PQSetClientEncoding(con : PGConn; encoding : CString) : Integer;
asmname '_p_pqsetclientencoding';

function PQSetDb(	host, port, options, tty, dbname : CSTRING) : PGConn;
asmname '_p_pqsetdb';

function PQSetDbLogin(host, port, options, tty, dbname, 
											login, passwd : CSTRING) : PGConn;
asmname '_p_pqsetdblogin';

{ Documentation is wrong. this function has _two_ arguments.
function always returns 0 (whatever arg is set to) }
function PQSetNonblocking(con : PGConn; arg : Integer) : Integer;
asmname '_p_pqsetnonblocking';

function PQSocket(con : PGConn) : Integer;
asmname '_p_pq_socket';

function PQStatus(con : PGConn) : Integer;
asmname '_p_pqstatus';

function PQTty(con : PGConn) : PGString;

{ Starts tracing session. See Example "tracetest.pas" }
procedure PQTrace(con : PGconn; var aFile : Text);

{ Ends tracing session. See Example "tracetest.pas" }
procedure PQUnTrace(con : PGconn);
asmname '_p_pquntrace';

function PQUser(con : PGConn) : PGString;


{ *************************************************** }
{ **************** IMPLEMENTATION ******************* }
{ *************************************************** }

implementation

{$L postgresc.c,crypt,pq}

{--------------------------------------}
{ ----- Original from C -------------- }
{--------------------------------------}

function CLoClose(conn:PGConn; fd:Integer):Integer;
asmname 'lo_close';

function CLoRead(conn:PGConn; fd:Integer; buf:PChar; len:Integer):Integer;
asmname 'lo_read';

function CLoLseek(conn:PGConn; fd,offset,whence:Integer):Integer;
asmname 'lo_lseek';

function CLoTell(conn:PGConn; fd:Integer):Integer;
asmname 'lo_tell';

function CLoWrite(conn:PGConn; fd:Integer; buf:PChar; len:Integer):Integer;
asmname 'lo_write';

function CPQBinaryTuples(res : PGResult) : Integer;
asmname '_p_pqbinary_tuples';

function CPQCmdStatus(res : PGResult) : CString;
asmname '_p_pqcmdstatus';

function CPQCmdTuples(res : PGResult) : CString;
asmname '_p_pqcmdtuples';

function CPQConsumeInput(con : PGConn) : Integer;
asmname '_p_pq_consume_input';

function CPQDb(con : PGConn) : CSTRING;
asmname '_p_pqdb';

procedure CPQDisplayTuples(res : PGResult; fd : Integer;
													fillAlign : Integer; fieldSep : CString;
													printHeader, quiet : Integer);
asmname '_p_pq_display_tuples';

function CPQErrorMessage(con : PGConn) : CString;
asmname '_p_pqerror_message';

function CPQFlush(con : PGConn) : Integer;
asmname '_p_pqflush';

function CPQFName(res : PGResult; findex : Integer) : CString;
asmname '_p_pqfname';

function CPQGetIsNull(res : PGResult; tupNum, fieldNum : Integer) : Integer;
asmname '_p_pqgetisnull';

function CPQGetValue(res : PGResult; tupNum, fieldNum : Integer) : CSTRING;
asmname '_p_pqgetvalue';

function CPQHost(con : PGConn) : CSTRING;
asmname '_p_pqhost';

function CPQIsBusy(con : PGConn) : Integer;
asmname '_p_pq_is_busy';

function CPQIsNonblocking(con : PGConn) : Integer;
asmname '_p_pqisnonblocking';

function CPQNotifies(con : PGConn) : Pointer;
asmname '_p_pq_notifies';

function CPQOidStatus(res : PGResult) : CString;
asmname '_p_pqoidstatus';

function CPQOptions(con : PGConn) : CSTRING;
asmname '_p_pqoptions';

function CPQPass(con : PGConn) : CSTRING;
asmname '_p_pqpass';

function CPQPort(con : PGConn) : CSTRING;
asmname '_p_pqport';

procedure CPQPrint(fd : Integer; res : PGResult; ppo : Pointer);
asmname '_p_pq_print';

procedure CPQPrintTuples(res : PGResult; fd : Integer;
													PrintAttrNAme, TerseOutput, Width : Integer);
asmname '_p_pq_print_tuples';

function CPQRequestCancel(con : PGconn) : Integer;
asmname '_p_pq_request_cancel';

function CPQResetStart(con : PGConn) : Integer;
asmname '_p_pqreset_start';

function CPQResultErrorMessage(res : PGResult) : CString;
asmname '_p_pqresulterrormessage';

function CPQResStatus(status : Integer) : CString;
asmname '_p_pqresstatus';

function CPQSendQuery(con : PGConn; query : CString) : Integer;
asmname '_p_pq_send_query';

function CPQTty(con : PGConn) : CSTRING;
asmname '_p_pqtty';

procedure CPQTrace(con : PGconn; fd : Integer);
asmname '_p_pqtrace';

function CPQUser(con : PGConn) : CSTRING;
asmname '_p_pquser';


{--------------------------------------}
{ -- Functions, which differ from C -- }
{--------------------------------------}

function LoClose(conn:PGConn; var f:AnyFile):Integer;
var
  fd:Integer; { C File Handle }
begin
  fd := FileHandle(f);
  LoClose := CLoClose(conn,fd)
end;

function LoRead(conn:PGConn; var f:AnyFile; buf:PChar; len:Integer):Integer;
var
  fd:Integer; { C File Handle }
begin
  fd := FileHandle(f);
  LoRead := CLoRead(conn,fd,buf,len)
end;

function LoLseek(conn:PGConn; var f:AnyFile; offset,whence:Integer):Integer;
var
  fd:Integer; { C File Handle }
begin
  fd := FileHandle(f);
  LoLseek := CLoLseek(conn,fd,offset, whence)
end;

function LoTell(conn:PGConn; var f:AnyFile):Integer;
var
  fd:Integer; { C File Handle }
begin
  fd := FileHandle(f);
  LoTell := CLoTell(conn,fd)
end;

function LoWrite(conn:PGConn; var f:AnyFile; buf:PChar; len:Integer):Integer;
var
  fd:Integer; { C File Handle }
begin
  fd := FileHandle(f);
  LoWrite := CLoWrite(conn,fd, buf, len)
end;

function PQBinaryTuples(res : PGResult) : Boolean;
begin
	if CPQBinaryTuples(res) = 1 then PQBinaryTuples:=True
	else PQBinaryTuples:=False
end;

function PQCmdStatus(res : PGResult) : PGString;
begin
	PQCmdStatus := CString2String(CPQCmdStatus(res))
end;

function PQCmdTuples(res : PGResult) : PGString;
begin
	PQCmdtuples := CString2String(CPQCmdTuples(res))
end;

function PQConsumeInput(con : PGConn) : Boolean;
begin
	if CPQConsumeInput(con) = 1  then PQConsumeInput := True
	else PQConsumeInput := False;
end;

function PQDb(con : PGConn) : PGString;
begin
	PQDb := CString2String(CPQDb(con))
end;

procedure PQDisplayTuples(res : PGResult; var aFile : Text; 
													fillAlign : Boolean; fieldSep : CString;
													printHeader, quiet : Boolean);
var
	fd, f, p, q : Integer;
begin
	fd := FileHandle(aFile);
	if fillAlign then f := 1 else f :=0;
	if printHeader then p := 1 else p :=0;
	if quiet then q := 1 else q :=0;
	CPQDisplayTuples(res, fd, f, fieldSep, p, q)
end;

function PQErrorMessage(con : PGConn) : PGString;
begin
	PQErrorMessage := CString2String(CPQErrorMessage(con))
end;

function PQFlush(con : PGConn) : Boolean;
begin
  if CPQFlush(con) = 0 then PQFlush := True
  else PQFlush := False
end;

function PQFName(res : PGResult; findex : Integer) : PGString;
begin
	PQFName := CString2String(CPQFName(res, findex))
end;

function PQGetIsNull(res : PGResult; tupNum, fieldNum : Integer) : Boolean;
begin
	if CPQGetIsNull(res, tupNum, fieldNum) = 1 then PQGetIsNull :=True
	else PQGetIsNull :=False
end;

function PQGetValue(res : PGResult; tupNum, fieldNum : Integer) : PGString;
begin
	PQGetValue := CString2String(CPQGetValue(res, tupNum, fieldNum))
end;

function PQHost(con : PGConn) : PGString;
begin
	PQHost := CString2String(CPQHost(con))
end;

function PQIsBusy(con : PGConn) : Boolean;
begin
	if CPQIsBusy(con) = 1 then PQIsBusy := True
	else PQIsBusy := False
end;

function PQIsNonblocking(con : PGConn) : Boolean;
begin
  if CPQIsNonblocking(con) = 1 then PQIsNonblocking := True
  else PQIsNonblocking := False
end;

function PQNotifies(con : PGConn) : PPGNotify;
Type		{ Just for internal use, for C-Interface }
	OPPGNotify = ^OPGNotify;
	OPGNotify = record
		r : Array[0..NAMEDATALEN-1] Of Char;
		b : Integer;
	end;
var
	oppgn : OPPGNotify;
	ppgn : PPGNotify;
begin
	oppgn := CPQNotifies(con);
	if oppgn <> nil then 
	begin
		ppgn := new (PPGNotify);
		ppgn^.relname := CString2String(oppgn^.r);
		ppgn^.be_pid  := oppgn^.b;
		PQNotifies  := ppgn
	end
	else PQNotifies := nil
end;

function PQOidStatus(res : PGResult) : PGString;
begin
	PQOidStatus := CString2String(CPQOidStatus(res))
end;

function PQOptions(con : PGConn) : PGSTRING;
begin
	PQOptions := CString2String(CPQOptions(con))
end;

function PQPass(con : PGConn) : PGString;
begin
	PQPass := CString2String(CPQPass(con))
end;

function PQPort(con : PGConn) : PGString;
begin
	PQPort := CString2String(CPQPort(con))
end;

procedure PQPrint(var aFile : Text; res : PGResult; ppo : PPQPrintOpt);
Type		{ Just for internal use to the C-Interface }
	OPPQPrintOpt = ^OPQPrintOpt;
	OPQPrintOpt =record
		header : Byte;
		align, standard, html3, expanded, pager : Byte;
		fieldSep, tableOpt, caption : CString;
		fieldName : Pointer;
	end;
var
	fd : Integer;
	oppo : OPPQPrintOpt;
begin
	fd := FileHandle(aFile);
	oppo := new (OPPQPrintOpt);
	with oppo^ do
	begin
		if ppo^.header		then header := 1 		else header :=0;
		if ppo^.align			then align :=1			else align :=0;
		if ppo^.standard	then standard :=1		else standard :=0;
		if ppo^.html3			then html3 :=1			else html3 :=0;
		if ppo^.expanded	then expanded :=1		else expanded:=0;
		if ppo^.pager			then pager:=1				else pager :=0;
		fieldSep :=ppo^.fieldSep;
		tableOpt:=ppo^.tableOpt;
		caption := ppo^.caption;
		fieldName := fieldName
	end;
	CPQPrint(fd, res, oppo)
end;

procedure PQPrintTuples(res : PGResult; var aFile : Text;
													PrintAttrName, TerseOutput : Boolean;
													Width : Integer);
var
	 p, t, fd : Integer;
begin
	fd := FileHandle(aFile);
	if PrintAttrName then p := 1 else p :=0;
	if TerseOutput then t := 1 else t :=0;
	CPQPrintTuples(res, fd, p, t, Width)
end;

function PQRequestCancel(con : PGconn) : Boolean;
begin
	if CPQRequestCancel(con) = 1 then PQRequestCancel := True
	else PQRequestCancel := False
end;

function PQResetStart(con : PGConn) : Boolean;
begin
	if CPQResetStart(con) = 0 then PQResetStart := False
	else PQResetStart := True
end;

function PQResStatus(status : Integer) : PGLongString;
begin
	PQResStatus := CString2String(CPQResStatus(status))
end;

function PQResultErrorMessage(res : PGResult) : PGString;
begin
	PQResultErrorMessage := CString2String(CPQResultErrorMessage(res))
end;

function PQSendQuery(con : PGConn; query : CString) : Boolean;
begin
	if CPQSendQuery(con, query) = 1 then PQSendQuery := True
	else PQSendQuery := False
end;

function PQTty(con : PGConn) : PGString;
begin
	PQTty := CString2String(CPQTty(con))
end;

procedure PQTrace(con : PGconn; var aFile : Text);
var
	fd : Integer;
begin
	fd := FileHandle(aFile);
	CPQTrace(con, fd)	
end;

function PQUser(con : PGConn) : PGString;
begin
	PQUser := CString2String(CPQUser(con))
end;



end.
