/*
Copyright (C) 2000 Eike Lange

Author: Eike Lange <eike.lange@uni-essen.de>

Support routine for postgres-unit

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation, version 2.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.

You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA.
*/

/* You may want to change this #include line. if your libpq-fe.h file
is in /usr/local/include/postgresql or /usr/include/postgresql, you
don't need to ;-) Else, you may remove "postgresql/" from this statement */
#include <postgresql/libpq-fe.h>

#include <stdio.h>
#include <unistd.h>

#include <errno.h> 

/*
 This is a global variable for 
 _p_pqtrace and _p_pquntrace
 Do not use it for other funcs!!!
*/
FILE *just_for_trace_routines;


/* ** Prototypes ** */

int					_p_pqbackend_pid(PGconn *con);
int					_p_pqbinary_tuples(PGresult *res);

void				_p_pqclear(PGresult *res);
int         _p_pqclientencoding(PGconn * con);
char *			_p_pqcmdstatus(PGresult *res);
const char *_p_pqcmdtuples(PGresult *res);
PQconninfoOption *_p_pqconndefaults();
PGconn *		_p_pqconnect_db(const char *coninfo);
int         _p_pqconnect_poll(PGconn * con);
PGconn *    _p_pqconnect_start(const char *coninfo);
void        _p_pqconninfo_free(PQconninfoOption *co);
int					_p_pq_consume_input(PGconn *con);

char *			_p_pqdb(PGconn *con);
void				_p_pq_display_tuples(PGresult *res, int fd,
									int fillAlign, const char *fieldSep, int printHeader,
									int quiet);
									
int         _p_pqendcopy(PGconn *con);
int         _p_pqenv2encoding(void);
char *			_p_pqerror_message(PGconn *con);
PGresult *	_p_pqexec(PGconn *con, const char *sql);

void				_p_pqfinish(PGconn *con);
int         _p_pqflush(PGconn *con);
int					_p_pqfmod(PGresult *res, int findex);
char *			_p_pqfname(PGresult *res, int findex);
int					_p_pqfnumber(PGresult *res, char *field);
int					_p_pqfsize(PGresult *res, int findex);
Oid					_p_pqftype(PGresult *res, int fnum);

int					_p_pqgetisnull(PGresult *res, int tup_num, int field_num);
int					_p_pqgetlength(PGresult *res, int tup_num, int field_num);
int         _p_pqgetline(PGconn *con, char *s, int len);
int         _p_pqgetline_async(PGconn *con, char *buffer, int bufsize);

PGresult *	_p_pq_get_result(PGconn *con);
char *			_p_pqgetvalue(PGresult *res, int tup_num, int field_num);

char *			_p_pqhost(PGconn *con);

int					_p_pq_is_busy(PGconn *con);

PGresult *	_p_pqmake_empty_pgresult(PGconn *con, int status);
int         _p_pqmblen(const unsigned char *s, int encoding);
int					_p_pqnfields(PGresult *res);
PGnotify*		_p_pq_notifies(PGconn *con);
int					_p_pqntuples(PGresult *res);

const char *_p_pqoidstatus(PGresult *res);
Oid         _p_pqoidvalue(const PGresult *res);
char *			_p_pqoptions(PGconn *con);

char *			_p_pqpass(PGconn *con);
char *			_p_pqport(PGconn *con);
void				_p_pq_print(int fd, PGresult *res, PQprintOpt *po);
void				_p_pq_print_tuples(PGresult *res, int fd, 
									int printAttrName, int terseOutput, int width);
int         _p_pqputline(PGconn *con, const char *s);
int         _p_pqputnbytes(PGconn *con, char *buffer, int buflen);

int					_p_pq_request_cancel(PGconn *con);
void				_p_pqreset(PGconn *con);
int         _p_pqreset_poll(PGconn *con);
int         _p_pqreset_start(PGconn *con);
char *      _p_pqresstatus(int status);
int					_p_pqresultstatus(PGresult *res);
const char *_p_pqresulterrormessage(PGresult *res);

int					_p_pq_send_query(PGconn *con, const char *query);
int         _p_pqsetclientencoding(PGconn * con, const char *encoding);
PGconn * 		_p_pqsetdb(const char *host, const char *pgport, 
														const char *options, const char *tty,
														const char *dbname);
PGconn * 		_p_pqsetdblogin(const char *host, const char *pgport, 
														const char *options, const char *tty,
														const char *dbname, const char *login,
														const char *passwd);

int         _p_pqsetnonblocking(PGconn *con, int arg);
int					_p_pq_socket(PGconn *con);
int					_p_pqstatus(PGconn *con);

char *			_p_pqtty(PGconn *con);
void        _p_pqtrace(PGconn *con, int fd);
void        _p_pquntrace(PGconn *con);
char *			_p_pquser(PGconn *con);





/* ** Functions ** */

int					_p_pqbackend_pid(PGconn *con)
{
	return PQbackendPID(con);
}

int					_p_pqbinary_tuples(PGresult *res)
{
	return PQbinaryTuples(res);
}

void				_p_pqclear(PGresult *res)
{
	return PQclear(res);
}

int _p_pqclientencoding(PGconn * con)
{
  return PQclientEncoding(con);
}

char *			_p_pqcmdstatus(PGresult *res)
{
	return PQcmdStatus(res);
}

const char *_p_pqcmdtuples(PGresult *res)
{
	return PQcmdTuples(res);
}

PQconninfoOption *_p_pqconndefaults()
{
	return PQconndefaults();
}

PGconn * _p_pqconnect_db(const char *coninfo)
{
	return PQconnectdb(coninfo);
}

int _p_pqconnect_poll(PGconn * con)
{
	return (int) PQconnectPoll(con);
}

PGconn * _p_pqconnect_start(const char *coninfo)
{
	return PQconnectStart(coninfo);
}

void _p_pqconninfo_free(PQconninfoOption *co)
{
  PQconninfoFree(co);
}

int					_p_pq_consume_input(PGconn *con)
{
	return PQconsumeInput(con);
}


char *			_p_pqdb(PGconn *con)
{
	return PQdb(con);
}

void _p_pq_display_tuples(PGresult *res, int fd, int fillAlign, 
                          const char *fieldSep, int printHeader,
                          int quiet)
{
	FILE *fil;
	fil=fdopen(dup(fd), "w");
	if(! fil) printf("ERROR in _p_pq_display_tuples: %s\n", strerror(errno));
	else PQdisplayTuples(res, fil, fillAlign, fieldSep, printHeader, quiet);
	fflush(fil);
	fclose(fil);
}

int _p_pqendcopy(PGconn *con)
{
  return PQendcopy(con);
}

int _p_pqenv2encoding(void)
{
  return PQenv2encoding();
}

char *			_p_pqerror_message(PGconn *con)
{
	return PQerrorMessage(con);
}

PGresult *	_p_pqexec(PGconn *con, const char *sql)
{
	return PQexec(con, sql);
}

void				_p_pqfinish(PGconn *con)
{
	PQfinish(con);
}

int _p_pqflush(PGconn *con)
{
  return PQflush(con);
}

int					_p_pqfmod(PGresult *res, int findex)
{
	return PQfmod(res, findex);
}


char *			_p_pqfname(PGresult *res, int findex)
{
	return PQfname(res, findex);
}

int					_p_pqfnumber(PGresult *res, char *field)
{
	return PQfnumber(res, field);
}

int					_p_pqfsize(PGresult *res, int findex)
{
	return PQfsize(res, findex);
}

Oid					_p_pqftype(PGresult *res, int fnum)
{
	return PQftype(res, fnum);
}

int					_p_pqgetisnull(PGresult *res, int tup_num, int field_num)
{
	return PQgetisnull(res, tup_num, field_num);
}

int					_p_pqgetlength(PGresult *res, int tup_num, int field_num)
{
	return PQgetlength(res, tup_num, field_num);
}

int _p_pqgetline(PGconn *con, char *s, int len)
{
  return PQgetline(con, s, len);
}

int _p_pqgetline_async(PGconn *con, char *buffer, int bufsize)
{
  return PQgetlineAsync(con, buffer, bufsize);
}


PGresult *	_p_pq_get_result(PGconn *con)
{
	return PQgetResult(con); 
}

char *			_p_pqgetvalue(PGresult *res, int tup_num, int field_num)
{
	return PQgetvalue(res, tup_num, field_num);
}

char *			_p_pqhost(PGconn *con)
{
	return PQhost(con);
}

int					_p_pq_is_busy(PGconn *con)
{
	/* 	don't know how TRUE and FALSE are actually 
			defined so I would like to use the safe way
	*/
	int _ret = PQisBusy(con);
	if (_ret)
		return 1;
	else 
		return 0;
}

int _p_pqisnonblocking(PGconn *con)
{
	if (PQisnonblocking(con)) return 1; else return 0;
}

PGresult *	_p_pqmake_empty_pgresult(PGconn *con, int status)
{
	return PQmakeEmptyPGresult(con, (int) status);
}

int _p_pqmblen(const unsigned char *s, int encoding)
{
  return PQmblen(s, encoding);
}

int					_p_pqnfields(PGresult *res)
{
	return PQnfields(res);
}

PGnotify*		_p_pq_notifies(PGconn *con)
{
	return PQnotifies(con);
}

int					_p_pqntuples(PGresult *res)
{
	return PQntuples(res);
}

const char *_p_pqoidstatus(PGresult *res)
{
	return PQoidStatus(res);
}

Oid _p_pqoidvalue(const PGresult *res)
{
	return PQoidValue(res);
}

char *			_p_pqoptions(PGconn *con)
{
	return PQoptions(con);
}

char *			_p_pqpass(PGconn *con)
{
	return PQpass(con);
}

char *			_p_pqport(PGconn *con)
{
	return PQport(con);
}

void				_p_pq_print(int fd, PGresult *res, PQprintOpt *po)
{
	FILE *filli;
	filli=fdopen(dup(fd), "w");
	if(! filli) printf("ERROR in _p_pq_print: %s\n", strerror(errno));
	else PQprint(filli, res, po);
	fflush(filli);
	fclose(filli);
}

void				_p_pq_print_tuples(PGresult *res, int fd, 
									int printAttrName, int terseOutput, int width)
{
	FILE *filli;
	filli=fdopen(dup(fd), "w");
	if(! filli) printf("ERROR in _p_pq_print_tuples: %s\n", strerror(errno));
	else PQprintTuples(res, filli, printAttrName, terseOutput, width);
	fflush(filli);
	fclose(filli);
}

int _p_pqputline(PGconn *con, const char *s)
{
  return PQputline(con, s);
}

int _p_pqputnbytes(PGconn *con, char *buffer, int buflen)
{
  return PQputnbytes(con, buffer, buflen);
}


int					_p_pq_request_cancel(PGconn *con)
{
	int _ret = PQrequestCancel(con);
	if(_ret) return 1;
	else return 0;
}

void _p_pqreset(PGconn *con)
{
  PQreset(con);
}

int _p_pqreset_poll(PGconn *con)
{
  return (int) PQresetPoll(con);
}

int _p_pqreset_start(PGconn *con)
{
  return PQresetStart(con);
}

char * _p_pqresstatus(int status)
{
  return PQresStatus(status);
}

int _p_pqresultstatus(PGresult *res)
{
  return (int) PQresultStatus(res);
}

const char *_p_pqresulterrormessage(PGresult *res)
{
	return PQresultErrorMessage(res);
}

int					_p_pq_send_query(PGconn *con, const char *query)
{
	return PQsendQuery(con, query);
}

int _p_pqsetclientencoding(PGconn * con, const char *encoding)
{
  return PQsetClientEncoding(con, encoding);
}


PGconn * 		_p_pqsetdb(const char *host, const char *pgport, 
														const char *options, const char *tty,
														const char *dbname)
{
	return PQsetdbLogin(host, pgport, options, tty, dbname, NULL, NULL);
}
														

PGconn * 		_p_pqsetdblogin(const char *host, const char *pgport, 
														const char *options, const char *tty,
														const char *dbname, const char *login,
														const char *passwd)
{
	return PQsetdbLogin(host, pgport, options, tty, dbname, login, passwd);
}

int _p_pqsetnonblocking(PGconn *con, int arg)
{
	return PQsetnonblocking(con, arg);
}

int					_p_pq_socket(PGconn *con)
{
	return	PQsocket(con);
}

int					_p_pqstatus(PGconn *con)
{
	return	(int) PQstatus(con);
}

char *			_p_pqtty(PGconn *con)
{
	return PQtty(con);
}

void _p_pqtrace(PGconn *con, int fd)
{
	just_for_trace_routines=fdopen(dup(fd), "w");
	if(! just_for_trace_routines)
		 printf("ERROR in _p_pqtrace: %s\n", strerror(errno));
	else PQtrace(con, just_for_trace_routines);
}

void _p_pquntrace(PGconn *con)
{
	fflush(just_for_trace_routines); 
	fclose(just_for_trace_routines);
	PQuntrace(con);
}


char *			_p_pquser(PGconn *con)
{
	return PQuser(con);
}





